==> cds-services-utils-1.18.1-sources.jar: com/sap/cds/services/utils/environment/VcapServicesParser.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.utils.environment;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sap.cds.services.environment.ServiceBinding;

/**
 * Parser for Open Service Broker API service bindings
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class VcapServicesParser implements ServiceBinding  {

	public final static String VCAP_SERVICES = "VCAP_SERVICES";

	private static final Logger logger = LoggerFactory.getLogger(VcapServicesParser.class);
	private static final ObjectMapper mapper = new ObjectMapper();

	public static List<ServiceBinding> getServiceBindings(String vcapServices) {
		List<ServiceBinding> bindings = new ArrayList<>();

		if (vcapServices != null && vcapServices.trim().length() > 0) {
			try {
				JsonNode vcapNodes = mapper.readTree(vcapServices);

				// if the VCAP_SERVICES key is part of the JSON, use its value
				if(vcapNodes.has(VCAP_SERVICES)) {
					vcapNodes = vcapNodes.get(VCAP_SERVICES);
				}

				// for each service a field exists
				vcapNodes.fields().forEachRemaining(serviceField -> {

					// for each binding an entry in the array exists
					Iterator<JsonNode> bindingNodes = serviceField.getValue().elements();
					int i = 0;
					while(bindingNodes.hasNext()) {
						try {
							VcapServicesParser binding = mapper.treeToValue(bindingNodes.next(), VcapServicesParser.class);
							binding.setService(serviceField.getKey());
							bindings.add(binding);
						} catch (IOException e) {
							logger.warn("Could not parse service binding of service '{}' at index ''.", serviceField.getKey(), i, e);
						}
						++i;
					}
				});
			} catch (IOException e) {
				logger.warn("Could not parse " + VCAP_SERVICES, e);
			}
		}

		return bindings;
	}

	private String service;

	@JsonProperty("name")
	private String name;

	@JsonProperty("plan")
	private String servicePlan;

	@JsonProperty("tags")
	private List<String> tags;

	@JsonProperty("credentials")
	private Map<String, Object> credentials;

	@Override
	public String getName() {
		return name;
	}

	@Override
	public String getService() {
		return service;
	}

	private void setService(String service) {
		this.service = service;
	}

	@Override
	public String getServicePlan() {
		return servicePlan;
	}

	@Override
	public List<String> getTags() {
		return tags; // NOSONAR
	}

	@Override
	public Map<String, Object> getCredentials() {
		return credentials;
	}

}

==> cds-services-utils-2.4.1-sources.jar: com/sap/cds/services/utils/environment/VcapServicesParser.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.utils.environment;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sap.cloud.environment.servicebinding.api.DefaultServiceBinding;
import com.sap.cloud.environment.servicebinding.api.ServiceBinding;

/**
 * Parser for Open Service Broker API service bindings
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class VcapServicesParser {

	public final static String VCAP_SERVICES = "VCAP_SERVICES";

	private static final Logger logger = LoggerFactory.getLogger(VcapServicesParser.class);
	private static final ObjectMapper mapper = new ObjectMapper();

	public static List<ServiceBinding> getServiceBindings(String vcapServices) {
		List<ServiceBinding> bindings = new ArrayList<>();

		if (vcapServices != null && vcapServices.trim().length() > 0) {
			try {
				JsonNode vcapNodes = mapper.readTree(vcapServices);

				// if the VCAP_SERVICES key is part of the JSON, use its value
				if(vcapNodes.has(VCAP_SERVICES)) {
					vcapNodes = vcapNodes.get(VCAP_SERVICES);
				}

				// for each service a field exists
				vcapNodes.fields().forEachRemaining(serviceField -> {

					// for each binding an entry in the array exists
					Iterator<JsonNode> bindingNodes = serviceField.getValue().elements();
					int i = 0;
					while(bindingNodes.hasNext()) {
						try {
							VcapServicesParser parseBinding = mapper.treeToValue(bindingNodes.next(), VcapServicesParser.class);
							ServiceBinding binding = DefaultServiceBinding.builder().copy(Collections.emptyMap())
									.withName(parseBinding.getName()).withServiceName(serviceField.getKey())
									.withServicePlan(parseBinding.servicePlan)
									.withTags(parseBinding.tags != null ? parseBinding.tags : Collections.emptyList())
									.withCredentials(parseBinding.credentials != null ? parseBinding.credentials : Collections.emptyMap())
									.build();
							bindings.add(binding);
						} catch (IOException e) {
							logger.warn("Could not parse service binding of service '{}' at index '{}'.", serviceField.getKey(), i, e);
						}
						++i;
					}
				});
			} catch (IOException e) {
				logger.warn("Could not parse " + VCAP_SERVICES, e);
			}
		}

		return bindings;
	}

	@JsonProperty("name")
	private String name;

	@JsonProperty("binding_name")
	private String bindingName;

	@JsonProperty("instance_name")
	private String instanceName;

	@JsonProperty("plan")
	private String servicePlan;

	@JsonProperty("tags")
	private List<String> tags;

	@JsonProperty("credentials")
	private Map<String, Object> credentials;

	private String getName() {
		if (name != null && !name.isEmpty()) {
			return name;
		} else if (bindingName != null && !bindingName.isEmpty()) {
			return bindingName;
		} else if (instanceName != null && !instanceName.isEmpty()) {
			return instanceName;
		}
		return null;
	}

}
